@@ -20,21 +20,18 @@
  * Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "gl_list.h"
+
 #include "db_storage.h"
 
 struct page_description {
 	char *name;
 	char *whatis;
-	struct page_description *next;
 };
 
-struct ult_trace;
-
-extern struct page_description *parse_descriptions (const char *base,
-						    const char *whatis);
-extern void store_descriptions (MYDBM_FILE dbf,
-				const struct page_description *head,
+/* Returns a list of struct page_description. */
+extern gl_list_t parse_descriptions (const char *base, const char *whatis);
+extern void store_descriptions (MYDBM_FILE dbf, gl_list_t descs,
 				struct mandata *info,
 				const char *path, const char *base,
-				struct ult_trace *trace);
-extern void free_descriptions (struct page_description *head);
+				gl_list_t trace);
