@@ -39,7 +39,7 @@
  * Wed May  4 15:44:47 BST 1994 Wilf. (G.Wilford@ee.surrey.ac.uk): changes
  * to get_dirlist() and manpath().
  *
- * This whole code segment is unfriendly and could do with a complete 
+ * This whole code segment is unfriendly and could do with a complete
  * overhaul.
  */
 
@@ -60,11 +60,15 @@
 #include <string.h>
 #include <unistd.h>
 
+#include "attribute.h"
 #include "canonicalize.h"
+#include "error.h"
 #include "gl_array_list.h"
 #include "gl_linkedhash_list.h"
 #include "gl_xlist.h"
+#include "xalloc.h"
 #include "xgetcwd.h"
+#include "xstrndup.h"
 #include "xvasprintf.h"
 
 #include "gettext.h"
@@ -72,10 +76,13 @@
 
 #include "manconfig.h"
 
-#include "error.h"
+#include "appendstr.h"
 #include "cleanup.h"
+#include "debug.h"
+#include "fatal.h"
 #include "glcontainers.h"
 #include "security.h"
+#include "util.h"
 
 #include "manp.h"
 #include "globbing.h"
@@ -136,12 +143,11 @@ static const char *get_config (const char *key, enum config_flag flag)
 	const struct config_item *item;
 	char *cont = NULL;
 
-	GL_LIST_FOREACH_START (config, item)
+	GL_LIST_FOREACH (config, item)
 		if (flag == item->flag && STREQ (key, item->key)) {
 			cont = item->cont;
 			break;
 		}
-	GL_LIST_FOREACH_END (config);
 
 	return cont;
 }
@@ -196,21 +202,20 @@ gl_list_t get_sections (void)
 	gl_list_t sections;
 	enum config_flag flag;
 
-	GL_LIST_FOREACH_START (config, item) {
+	GL_LIST_FOREACH (config, item) {
 		if (item->flag == SECTION_USER)
 			length_user++;
 		else if (item->flag == SECTION)
 			length++;
-	} GL_LIST_FOREACH_END (config);
+	}
 	sections = new_string_list (GL_ARRAY_LIST, true);
 	if (length_user)
 		flag = SECTION_USER;
 	else
 		flag = SECTION;
-	GL_LIST_FOREACH_START (config, item)
+	GL_LIST_FOREACH (config, item)
 		if (item->flag == flag)
 			gl_list_add_last (sections, xstrdup (item->key));
-	GL_LIST_FOREACH_END (config);
 	return sections;
 }
 
@@ -281,7 +286,10 @@ static char *pathappend (char *oldpath, const char *appendage)
 			search = strstr (app_dedup, tok);
 			while (search) {
 				char *terminator = search + strlen (tok);
-				if (!*terminator) {
+				if (search > app_dedup && search[-1] != ':')
+					/* Ignore suffix matches. */
+					;
+				else if (!*terminator) {
 					/* End of the string, so chop here. */
 					*search = 0;
 					while (search > app_dedup &&
@@ -293,6 +301,7 @@ static char *pathappend (char *oldpath, const char *appendage)
 					*search = 0;
 					newapp = xasprintf ("%s%s", app_dedup,
 							    terminator + 1);
+					assert (newapp);
 					free (app_dedup);
 					app_dedup = newapp;
 				}
@@ -331,7 +340,7 @@ static void gripe_not_directory (const char *dir)
 		error (0, 0, _("warning: %s isn't a directory"), dir);
 }
 
-/* accept a manpath list, separated with ':', return the associated 
+/* accept a manpath list, separated with ':', return the associated
    catpath list */
 char *cat_manpath (char *manp)
 {
@@ -342,12 +351,12 @@ char *cat_manpath (char *manp)
 		catdir = get_config (path, MANDB_MAP_USER);
 		if (!catdir)
 			catdir = get_config (path, MANDB_MAP);
-		catp = catdir ? pathappend (catp, catdir) 
+		catp = catdir ? pathappend (catp, catdir)
 			      : pathappend (catp, path);
 	}
 
 	return catp;
-}		
+}
 
 /* Unpack a glibc-style locale into its component parts.
  *
@@ -667,7 +676,7 @@ static char *guess_manpath (const char *systems)
 			/* Things aren't going to work well here either... */
 			if (!quiet)
 				error (0, 0, _("warning: empty $PATH"));
-			
+
 			return add_system_manpath (systems,
 						   def_path (MANDATORY));
 		}
@@ -771,12 +780,12 @@ static void add_to_dirlist (FILE *config_file, int user)
 		else if (sscanf (bp, "MANBIN %*s") == 1)
 			goto next;
 		else if (sscanf (bp, "MANDATORY_MANPATH %511s", key) == 1)
-			add_mandatory (key);	
+			add_mandatory (key);
 		else if (sscanf (bp, "MANPATH_MAP %511s %511s",
-			 key, cont) == 2) 
+			 key, cont) == 2)
 			add_manpath_map (key, cont);
 		else if ((c = sscanf (bp, "MANDB_MAP %511s %511s",
-				      key, cont)) > 0) 
+				      key, cont)) > 0)
 			add_mandb_map (key, c == 2 ? cont : key, user);
 		else if ((c = sscanf (bp, "DEFINE %511s %511[^\n]",
 				      key, cont)) > 0)
@@ -805,7 +814,7 @@ next:
 	free (buf);
 }
 
-static void free_config_file (void *unused _GL_UNUSED)
+static void free_config_file (void *unused MAYBE_UNUSED)
 {
 	gl_list_free (config);
 }
@@ -872,13 +881,13 @@ static char *def_path (enum config_flag flag)
 	char *manpath = NULL;
 	const struct config_item *item;
 
-	GL_LIST_FOREACH_START (config, item)
+	GL_LIST_FOREACH (config, item)
 		if (item->flag == flag) {
 			gl_list_t expanded_dirs;
 			const char *expanded_dir;
 
 			expanded_dirs = expand_path (item->key);
-			GL_LIST_FOREACH_START (expanded_dirs, expanded_dir) {
+			GL_LIST_FOREACH (expanded_dirs, expanded_dir) {
 				int status = is_directory (expanded_dir);
 
 				if (status < 0)
@@ -891,10 +900,9 @@ static char *def_path (enum config_flag flag)
 				else if (status == 1)
 					manpath = pathappend (manpath,
 							      expanded_dir);
-			} GL_LIST_FOREACH_END (expanded_dirs);
+			}
 			gl_list_free (expanded_dirs);
 		}
-	GL_LIST_FOREACH_END (config);
 
 	/* If we have complete config file failure... */
 	if (!manpath)
@@ -961,7 +969,7 @@ char *get_manpath_from_path (const char *path, int mandatory)
 		/* If the directory we're working on has MANPATH_MAP entries
 		 * in the config file, add them to the list.
 		 */
-		GL_LIST_FOREACH_START (config, config_item) {
+		GL_LIST_FOREACH (config, config_item) {
 			if (MANPATH_MAP != config_item->flag ||
 			    !STREQ (p, config_item->key))
 				continue;
@@ -970,9 +978,9 @@ char *get_manpath_from_path (const char *path, int mandatory)
 			manpath_map_found = true;
 			insert_override_dir (tmplist, config_item->cont);
 			add_dir_to_list (tmplist, config_item->cont);
-		} GL_LIST_FOREACH_END (config);
+		}
 
-		 /* The directory we're working on isn't in the config file.  
+		 /* The directory we're working on isn't in the config file.
 		    See if it has ../man, man, ../share/man, or share/man
 		    subdirectories.  If so, and they haven't been added to
 		    the list, do. */
@@ -988,19 +996,18 @@ char *get_manpath_from_path (const char *path, int mandatory)
 	if (mandatory) {
 		debug ("adding mandatory man directories\n");
 
-		GL_LIST_FOREACH_START (config, config_item) {
+		GL_LIST_FOREACH (config, config_item) {
 			if (config_item->flag == MANDATORY) {
 				insert_override_dir (tmplist,
 						     config_item->key);
 				add_dir_to_list (tmplist, config_item->key);
 			}
-		} GL_LIST_FOREACH_END (config);
+		}
 	}
 
 	len = 0;
-	GL_LIST_FOREACH_START (tmplist, item)
+	GL_LIST_FOREACH (tmplist, item)
 		len += strlen (item) + 1;
-	GL_LIST_FOREACH_END (tmplist);
 
 	if (!len)
 		/* No path elements in configuration file or with
@@ -1012,12 +1019,12 @@ char *get_manpath_from_path (const char *path, int mandatory)
 	*manpathlist = '\0';
 
 	p = manpathlist;
-	GL_LIST_FOREACH_START (tmplist, item) {
+	GL_LIST_FOREACH (tmplist, item) {
 		len = strlen (item);
 		memcpy (p, item, len);
 		p += len;
 		*p++ = ':';
-	} GL_LIST_FOREACH_END (tmplist);
+	}
 
 	p[-1] = '\0';
 
@@ -1058,9 +1065,8 @@ static void add_dir_to_list (gl_list_t list, const char *dir)
 	const char *expanded_dir;
 
 	expanded_dirs = expand_path (dir);
-	GL_LIST_FOREACH_START (expanded_dirs, expanded_dir)
+	GL_LIST_FOREACH (expanded_dirs, expanded_dir)
 		add_expanded_dir_to_list (list, expanded_dir);
-	GL_LIST_FOREACH_END (expanded_dirs);
 	gl_list_free (expanded_dirs);
 }
 
@@ -1071,9 +1077,9 @@ static void add_man_subdirs (gl_list_t list, const char *path)
 {
 	char *newpath;
 
-	/* don't assume anything about path, especially that it ends in 
+	/* don't assume anything about path, especially that it ends in
 	   "bin" or even has a '/' in it! */
-	   
+
 	char *subdir = strrchr (path, '/');
 	if (subdir) {
 		newpath = xasprintf ("%.*s/man", (int) (subdir - path), path);
@@ -1128,14 +1134,14 @@ static struct canonicalized_path *canonicalized_path_new (const char *path)
 	return cp;
 }
 
-static bool _GL_ATTRIBUTE_PURE canonicalized_path_equals (const void *elt1,
-							  const void *elt2)
+static bool ATTRIBUTE_PURE canonicalized_path_equals (const void *elt1,
+						      const void *elt2)
 {
 	const struct canonicalized_path *cp1 = elt1, *cp2 = elt2;
 	return string_equals (cp1->canon_path, cp2->canon_path);
 }
 
-static size_t _GL_ATTRIBUTE_PURE canonicalized_path_hash (const void *elt)
+static size_t ATTRIBUTE_PURE canonicalized_path_hash (const void *elt)
 {
 	const struct canonicalized_path *cp = elt;
 	return string_hash (cp->canon_path);
@@ -1158,7 +1164,7 @@ static void add_dir_to_path_list (gl_list_t list, const char *p)
 	char *expanded_dir;
 
 	expanded_dirs = expand_path (p);
-	GL_LIST_FOREACH_START (expanded_dirs, expanded_dir) {
+	GL_LIST_FOREACH (expanded_dirs, expanded_dir) {
 		int status = is_directory (expanded_dir);
 
 		if (status < 0)
@@ -1173,8 +1179,8 @@ static void add_dir_to_path_list (gl_list_t list, const char *p)
 			if (*expanded_dir != '/') {
 				char *cwd = xgetcwd ();
 				if (!cwd)
-					error (FATAL, errno,
-							_("can't determine current directory"));
+					fatal (errno,
+					       _("can't determine current directory"));
 				path = appendstr (cwd, "/", expanded_dir,
 						  (void *) 0);
 			} else
@@ -1188,7 +1194,7 @@ static void add_dir_to_path_list (gl_list_t list, const char *p)
 				canonicalized_path_free (cp);
 			free (path);
 		}
-	} GL_LIST_FOREACH_END (expanded_dirs);
+	}
 	gl_list_free (expanded_dirs);
 }
 
@@ -1220,18 +1226,17 @@ gl_list_t create_pathlist (const char *manp)
 	}
 
 	list = new_string_list (GL_ARRAY_LIST, false);
-	GL_LIST_FOREACH_START (canonicalized_list, cp)
+	GL_LIST_FOREACH (canonicalized_list, cp)
 		gl_list_add_last (list, xstrdup (cp->path));
-	GL_LIST_FOREACH_END (canonicalized_list);
 
 	if (debug_level) {
 		debug ("final search path = ");
-		GL_LIST_FOREACH_START (list, p) {
+		GL_LIST_FOREACH (list, p) {
 			if (!gl_list_previous_node (list, list_node))
 				debug ("%s", p);
 			else
 				debug (":%s", p);
-		} GL_LIST_FOREACH_END (list);
+		}
 		debug ("\n");
 	}
 
@@ -1250,10 +1255,9 @@ char *get_mandb_manpath (void)
 	char *manpath = NULL;
 	const struct config_item *item;
 
-	GL_LIST_FOREACH_START (config, item)
+	GL_LIST_FOREACH (config, item)
 		if (item->flag == MANDB_MAP || item->flag == MANDB_MAP_USER)
 			manpath = pathappend (manpath, item->key);
-	GL_LIST_FOREACH_END (config);
 
 	return manpath;
 }
@@ -1263,8 +1267,8 @@ char *get_mandb_manpath (void)
  * Return catdir mapping or NULL if it isn't a global/user mandir (as
  * appropriate).
  *
- * This routine would seem to work correctly for nls subdirs and would 
- * specify the (correct) consistent catpath even if not defined in the 
+ * This routine would seem to work correctly for nls subdirs and would
+ * specify the (correct) consistent catpath even if not defined in the
  * config file.
  *
  * Do not return user catpaths when cattype == 0! This is used to decide
@@ -1276,7 +1280,7 @@ char *get_catpath (const char *name, int cattype)
 	const struct config_item *item;
 	char *ret = NULL;
 
-	GL_LIST_FOREACH_START (config, item)
+	GL_LIST_FOREACH (config, item)
 		if (((cattype & SYSTEM_CAT) && item->flag == MANDB_MAP) ||
 		    ((cattype & USER_CAT)   && item->flag == MANDB_MAP_USER)) {
 			size_t manlen = strlen (item->key);
@@ -1321,7 +1325,6 @@ char *get_catpath (const char *name, int cattype)
 				break;
 			}
 		}
-	GL_LIST_FOREACH_END (config);
 
 	return ret;
 }
@@ -1334,25 +1337,24 @@ bool is_global_mandir (const char *dir)
 	const struct config_item *item;
 	bool ret = false;
 
-	GL_LIST_FOREACH_START (config, item)
+	GL_LIST_FOREACH (config, item)
 		if (item->flag == MANDB_MAP &&
 		    STRNEQ (dir, item->key, strlen (item->key))) {
 		    	ret = true;
 			break;
 		}
-	GL_LIST_FOREACH_END (config);
 
 	return ret;
 }
 
-/* Accept a manpath (not a full pathname to a file) and return an FSSTND 
+/* Accept a manpath (not a full pathname to a file) and return an FSSTND
    equivalent catpath */
 static char *fsstnd (const char *path)
 {
 	char *manpath;
 	char *catpath;
 	char *element;
-	
+
 	if (strncmp (path, MAN_ROOT, sizeof MAN_ROOT - 1) != 0) {
 		if (!quiet)
 			error (0, 0, _("warning: %s does not begin with %s"),
@@ -1364,19 +1366,19 @@ static char *fsstnd (const char *path)
 	manpath = xstrdup (path);
 	catpath = xmalloc (strlen (path) + sizeof CAT_ROOT - 3);
 
-	/* start with CAT_ROOT */ 
+	/* start with CAT_ROOT */
 	(void) strcpy (catpath, CAT_ROOT);
 
 	/* split up path into elements and deal with accordingly */
 	for (element = strtok (manpath, "/"); element;
 	     element = strtok (NULL, "/")) {
 		if (strncmp (element, "man", 3) == 0) {
-			if (*(element + 3)) { 
+			if (*(element + 3)) {
 				*element = 'c';
 				*(element + 2) = 't';
 			} else
 				continue;
-		} 
+		}
 		(void) strcat (catpath, "/");
 		(void) strcat (catpath, element);
 	}
